package cliente;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import exceptions.PacoteCorrompidoException;

public class PistaDecolagem implements Runnable{
	private DatagramSocket clientSocket;
	private byte[] receiveData;
	private DatagramPacket resposta;

	public DatagramPacket getResposta() {
		return resposta;
	}

	public PistaDecolagem() throws IOException{
		this.clientSocket = new DatagramSocket(20001);
		this.receiveData = new byte[13000];
		this.resposta = new DatagramPacket(this.receiveData, this.receiveData.length);		
	}

	public void close(){
		clientSocket.close();
	}
	
	public DatagramPacket Enviar(DatagramPacket pedido) throws IOException, PacoteCorrompidoException{
		byte operacao = pedido.getData()[0];
		byte[] tamanho = longToBytes(pedido.getData().length+1);
		byte[] checksum = longToBytes(contagemChecksum(pedido.getData()));
		byte dados[] = new byte[pedido.getData().length+16];

		for (int i = 0; i < dados.length; i++) {
			if(i<8){
				dados[i] = tamanho[i];
			}else if(i<16){
				dados[i] = checksum[i-8];
			}else{
				dados[i] = pedido.getData()[i-16];
			}
		}
		pedido = new DatagramPacket(dados, dados.length, pedido.getAddress(), pedido.getPort());
		
		clientSocket.send(pedido); // o pacote foi enviado, agora se espera a resposta.
		
		this.clientSocket.receive(this.resposta);
		
		tamanho = new byte[8];
		checksum = new byte[8];

		for (int i = 0; i < 16; i++) {
			if(i<8){
				tamanho[i] = this.resposta.getData()[i];
			}else{
				checksum[i-8] = this.resposta.getData()[i];
			}
		}
		long tamanhoDados = getLong(tamanho, 0, 8);
		long checksumL = getLong(checksum, 0, 8);
		dados = new byte[(int) tamanhoDados];

		for (int i = 17; i < tamanhoDados+17; i++) {
			dados[i-17] = this.resposta.getData()[i];
		}

		if(!(checksumL == contagemChecksum(dados))){
			throw new PacoteCorrompidoException();
		}else if(operacao != this.resposta.getData()[16]){
			throw new PacoteCorrompidoException();
		}else{
			pedido = new DatagramPacket(dados, dados.length, pedido.getAddress(), pedido.getPort());
		}
		
		System.out.println(this.resposta);
		return this.resposta;

	}

	private static long contagemChecksum(byte[] buf) {
		int length = buf.length;
		int i = 0;

		long sum = 0;
		long data;

		// Handle all pairs
		while (length > 1) {
			// Corrected to include @Andy's edits and various comments on Stack Overflow
			data = (((buf[i] << 8) & 0xFF00) | ((buf[i + 1]) & 0xFF));
			sum += data;
			// 1's complement carry bit correction in 16-bits (detecting sign extension)
			if ((sum & 0xFFFF0000) > 0) {
				sum = sum & 0xFFFF;
				sum += 1;
			}

			i += 2;
			length -= 2;
		}

		// Handle remaining byte in odd length buffers
		if (length > 0) {
			// Corrected to include @Andy's edits and various comments on Stack Overflow
			sum += (buf[i] << 8 & 0xFF00);
			// 1's complement carry bit correction in 16-bits (detecting sign extension)
			if ((sum & 0xFFFF0000) > 0) {
				sum = sum & 0xFFFF;
				sum += 1;
			}
		}

		// Final 1's complement value correction to 16-bits
		sum = ~sum;
		sum = sum & 0xFFFF;
		return sum;

	}

	private static byte[] longToBytes(long l) {  
		byte[] data = new byte[8];  
		for (int i=data.length-1; i >= 0; i--) {  
			data[i] = (byte) (l & 0xFF);  
			l >>= 8;  
		}  
		return data;  
	}  

	private static long getLong(byte[] data, int offset, int length) {  
		long l = 0;  
		int i = 0;  
		while (i < length) {  
			l <<= 8;  
			l += data[offset + i] & 0xFF;  
			i++;  
		}  
		return l;  
	}

	public void run() {
		/*
		while(true){
			try {
				System.out.println("estou esperando um pacote");
				this.clientSocket.receive(this.resposta);
				System.out.println("peguei um pacote");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}		
		} */
	}
}
