package interfaces;

import java.net.DatagramPacket;
import java.net.InetAddress;

public interface Servidor {
	public String[] usuariosCadastrados();
	public boolean[] statusUsuario();
	public InetAddress[] enderecoUsuario();
	public Integer[] portasUsuarios();
	public DatagramPacket requisicao(DatagramPacket pedido);
	public void guardarDados();
}

	/* O servidor podera receber varios tipos de pacotes, e dependendo desse tipo devera responder apropriadamente. 
	 * Para identificar o tipo o pacote deve ter um numero identificador.
	 * 
	 * A classe de recebimento dos pacotes fica responsavel por analisar e remover o checksum e o tamanho do pacote.
	 * A classe de envio dos pacotes fica responsavel por adicionar 1byte de operacao, o checksum e o tamanho do pacote
	 * 
	 * O 1 byte de cada cabecario deve ter sempre o codigo da operacao
	 **************************************************************************************************************************
	 * 0: Cadastro: String cadastro(String login, String senha)
	 * O sistema recebera o login e senha do usario e o adionara ao repositorio de usuarios, enviara depois uma mensagem de sucesso ou falha.
	 * 
	 * O pedido tem:
	 * O segundo byte tem o tamanho do login
	 * O terceiro byte tem o tamanho da senha
	 * Login
	 * Senha
	 * 
	 * A resposta tem:
	 * 1 byte com o tamanho da string
	 * Um string(Toda parte de dados sera um String) com a mensagem de sucesso ou erro.
	 * 
	 **************************************************************************************************************************
	 * 1: Login/atualizar lista de contatos:
	 * O sistema recebera o login, senha do usuario e o endereco dele. Respondera primeiramente com a lista de contatos
	 *  e um array de booleanos que indica os contatos online.
	 * Tambem pode servir com a utilidade de atualizar a lista de contatos e aqueles que estao online.
	 * O cliente devera fazer isso ao menos a cada um minuto, mas o usuario podera atualizar a lista por ele mesmo.
	 * 
	 * O pedido tem:
	 * O segundo byte tem o tamanho do login
	 * O terceiro byte tem o tamanho da senha
	 * Login
	 * Senha
	 * 
	 * A resposta tem:
	 * 1 byte representando se houve erro (0: tudo ok, 1: usuario nao cadastrado, 2: senha invalida)
	 * 1 byte com o tamanho do array que diz o tamanho do login de cada contato
	 * 1 array de byte com o tamanho do nome de cada contato
	 * 1 array de String com os nomes do contato (pode ser nulo)
	 * 1 array de booleanos mostrando os contatos online (pode ser nulo e na real serao bytes, 1 = true, 2 false)
	 **************************************************************************************************************************
	 * 2: Usuario iniciando conexao com outro usuario
	 * O sistema recebera  o login e senha do usuario e o login do contato. Se estiver disponivel devolvera o endereo de conexao do contato desejado.
	 * 
	 * O pedido tem:
	 * O segundo byte tem o tamanho do login
	 * O terceiro byte tem o tamanho da senha
	 * O quarto byte tem o tamnho do login do contato
	 * Login
	 * Senha
	 * Login do contato
	 * 
	 * A resposta tem:
	 * 4 bytes que serao o numero da porta do contato
	 * 4 bytes que serao o ip do contato
	 * 1 byte tamanho login do contato
	 * Login do contato
	 * 
	 **************************************************************************************************************************
	 * 3: Usuario adicionando contato
	 * O sistema recebe o login, senha e login do contato e adiciona o contato a lista de contatos do usuario e vice versa.
	 * 
	 * O pedido tem:
	 * O segundo byte tem o tamanho do login
	 * O terceiro byte tem o tamanho da senha
	 * O quarto byte tem o tamnho do login do contato
	 * Login
	 * Senha
	 * Login do contato
	 * 
	 * A resposta tem:
	 * 1 byte representando o erro.
	 * 
	 **************************************************************************************************************************
	 * 4: Usuario removendo contato
	 * similar a adicionar, mas remove.
	 **************************************************************************************************************************
	 * 5: Desconectar usuario
	 * O sistema recebe a senha e login do usuario deixa-o em offline.
	 * 
	 * O pedido tem:
	 * O segundo byte tem o tamanho do login
	 * O terceiro byte tem o tamanho da senha
	 * Login
	 * Senha
	 * 
	 * A resposta tem: 
	 * 1 byte representando o erro.
	 */
