package servidor;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;

import exceptions.PacoteCorrompidoException;

public class PistaPouso {
	private DatagramSocket serverSocket;

	public PistaPouso() {
		try {
			this.serverSocket = new DatagramSocket(20000);
		} catch (SocketException e) {
			e.printStackTrace();
		}
	}
	
	public void close(){
		serverSocket.close();
	}
	
	
	public DatagramPacket receber() throws IOException, PacoteCorrompidoException {
		byte[] receiveData = new byte[500];
		DatagramPacket pedido = new DatagramPacket(receiveData, receiveData.length);
		serverSocket.receive(pedido);
		byte[] tamanho = new byte[8];
		byte[] checksum = new byte[8];

		for (int i = 0; i < 16; i++) {
			if(i<8){
				tamanho[i] = pedido.getData()[i];
			}else{
				checksum[i-8] = pedido.getData()[i];
			}
		}
		long tamanhoDados = getLong(tamanho, 0, 8);
		long checksumL = getLong(checksum, 0, 8);
		byte[] dados = new byte[(int) tamanhoDados];

		for (int i = 16; i < tamanhoDados+16; i++) {
			dados[i-16] = pedido.getData()[i];
		}

		if(!(checksumL == contagemChecksum(dados))){
			throw new PacoteCorrompidoException();
		}else{
			pedido = new DatagramPacket(dados, dados.length, pedido.getAddress(), pedido.getPort());
		}

		return pedido;
	}

	public void enviar(DatagramPacket resposta, byte operacao) throws IOException {
		byte[] tamanho = longToBytes(resposta.getData().length+1);
		byte[] checksum = longToBytes(contagemChecksum(resposta.getData()));
		byte dados[] = new byte[resposta.getData().length+17];

		for (int i = 0; i < dados.length; i++) {
			if(i<8){
				dados[i] = tamanho[i];
			}else if(i<16){
				dados[i] = checksum[i-8];
			}else if(i==16){
				dados[i] = operacao;
			}else{
				dados[i] = resposta.getData()[i-17];
			}
		}
		resposta = new DatagramPacket(dados, dados.length, resposta.getAddress(), resposta.getPort());
		this.serverSocket.send(resposta);
	}

	private long contagemChecksum(byte[] buf) {
		int length = buf.length;
		int i = 0;

		long sum = 0;
		long data;

		// Handle all pairs
		while (length > 1) {
			// Corrected to include @Andy's edits and various comments on Stack Overflow
			data = (((buf[i] << 8) & 0xFF00) | ((buf[i + 1]) & 0xFF));
			sum += data;
			// 1's complement carry bit correction in 16-bits (detecting sign extension)
			if ((sum & 0xFFFF0000) > 0) {
				sum = sum & 0xFFFF;
				sum += 1;
			}

			i += 2;
			length -= 2;
		}

		// Handle remaining byte in odd length buffers
		if (length > 0) {
			// Corrected to include @Andy's edits and various comments on Stack Overflow
			sum += (buf[i] << 8 & 0xFF00);
			// 1's complement carry bit correction in 16-bits (detecting sign extension)
			if ((sum & 0xFFFF0000) > 0) {
				sum = sum & 0xFFFF;
				sum += 1;
			}
		}

		// Final 1's complement value correction to 16-bits
		sum = ~sum;
		sum = sum & 0xFFFF;
		return sum;

	}

	private static byte[] longToBytes(long l) {  
		byte[] data = new byte[8];  
		for (int i=data.length-1; i >= 0; i--) {  
			data[i] = (byte) (l & 0xFF);  
			l >>= 8;  
		}  
		return data;  
	}  

	private static long getLong(byte[] data, int offset, int length) {  
		long l = 0;  
		int i = 0;  
		while (i < length) {  
			l <<= 8;  
			l += data[offset + i] & 0xFF;  
			i++;  
		}  
		return l;  
	}  
}
