package servidor;

import java.io.File;
//import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
//import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.DatagramPacket;
import java.net.InetAddress;

import javax.swing.SwingUtilities;

import servidor.gui.ServidorGUI;

import dados.RepositorioUsuarios;
import dados.Usuario;

import exceptions.ContatoNaoEncontradoException;
import exceptions.JaCadastradoException;
import exceptions.JaEhContatoException;
import exceptions.NaoPodeAdicionarMaisException;
import exceptions.NaoPodeCadastrarMaisException;
import exceptions.SenhaInvalidaException;
import exceptions.UsuarioNaoEncontradoException;
import exceptions.PacoteCorrompidoException;

public class Servidor implements interfaces.Servidor{

	private RepositorioUsuarios repositorio;
	private String[] usuarios;
	private boolean[] usariosLogados;
	private InetAddress[] endereco;
	private Integer[] portas; //tem que ser Integer pra poder referenciar null
	private byte index;

	ServidorGUI gui;

	/*
	 * **********
	 * CONSTRUTOR
	 * **********
	 */

	public Servidor() {
		gui = new ServidorGUI();
		//FileInputStream fis;
		//recupera os contatos cadastrados
		//try {
		//fis = new FileInputStream(new File("dados.sky"));
		//ObjectInputStream dados = new ObjectInputStream(fis);
		//Usuario[] usuario = (Usuario[]) dados.readObject();

		this.repositorio = new RepositorioUsuarios();
		this.usuarios = new String[100];
		this.usariosLogados = new boolean[100];
		this.endereco = new InetAddress[100];
		this.portas = new Integer[100];
		this.index = 0;

		//for (; index < usuario.length && usuario[index]!=null; index++) {
		//	this.repositorio.inserirUsuario(usuario[index]);
		//}


		/*} catch (FileNotFoundException e) {
			this.repositorio = new RepositorioUsuarios();
			this.usuarios = new String[100];
			this.usariosLogados = new boolean[100];
			this.endereco = new InetAddress[100];
			this.portas = new int[100];
			this.index = 0;
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (JaCadastradoException e) {
			e.printStackTrace();
		} catch (NaoPodeCadastrarMaisException e) {
			e.printStackTrace();
		}*/
	}

	/*
	 * *************************
	 * METODOS DE ACESSO PRIVADO
	 * *************************
	 */

	@SuppressWarnings("unused")
	private String getUsuario(int num){
		return repositorio.getUsuario(num);
	}

	/**
	 * Retorna os contatos que, dentre os enviados no argumento<br>
	 * estao logados no servidor.
	 * @param meuscontatos : Contatos do usuario solicitante.
	 * @return
	 */
	private boolean[] getOnline(String[] meuscontatos){
		boolean[] retorno = new boolean[meuscontatos.length];
		for (int i = 0; i < meuscontatos.length; i++) {
			retorno[i]=false;
			for (int j = 0; j < this.index; j++) {
				if(meuscontatos[i].equals(this.usuarios[j])){
					retorno[i] = true;
				}
			}
		}
		return retorno;
	}

	/**
	 * Retorna endereco IP do contado enviado como parametro.
	 * @param contato : String que representa nome do usuario.
	 * @return {@link InetAddress}
	 */
	private InetAddress getIP(String contato){
		InetAddress retorno = null;
		for (int i = 0; i < this.index; i++) {
			if(contato.equals(this.usuarios[i])){
				retorno = this.endereco[i];
			}
		}
		return retorno;
	}

	//retorna a porta do usuario
	private int getPorta(String contato){
		int retorno = 0;
		for (int i = 0; i < this.index; i++) {
			if(contato.equals(this.usuarios[i])){
				retorno = this.portas[i];
			}
		}
		return retorno;	
	}

	//marca o usuario como online
	private void setOnline(String login, InetAddress endereco, int porta){
		if(endereco != null) {
			gui.adicionarEntrada(login, endereco.getCanonicalHostName(), "online");
		}
		else
			System.err.println("Inet null");
		for (int i = 0; i < this.index; i++) {
			if(login.equals(this.usuarios[i])){
				this.usariosLogados[i] = true;
				this.endereco[i] = endereco;
				this.portas[i] = porta;
			}
		}
	}

	//marca o usuario como offline
	private void setOffline(String login){
		gui.removerEntrada(login);
		for (int i = 0; i < this.index; i++) {
			if(login.equals(this.usuarios[i])){
				this.usariosLogados[i] = false;
				this.endereco[i]=null;
				this.portas[i] = (Integer) null; //tem que ser Integer pra poder referenciar null
			}
		}
	}

	//cadastra novos usuarios
	private void cadastrarUsuario(String login, String senha) throws JaCadastradoException, NaoPodeCadastrarMaisException{
		Usuario novo = new Usuario(login, senha);
		this.repositorio.inserirUsuario(novo);
		this.usuarios[this.index]=login;
		this.index+=1;
	}

	//adiciona novos contatos
	private void adicionarContato(String login, String contato) throws JaEhContatoException, NaoPodeAdicionarMaisException, UsuarioNaoEncontradoException{
		this.repositorio.adicionarContato(login, contato);
		this.repositorio.adicionarContato(contato, login);
	}

	//remove contato
	private void removerContato(String login, String contato) throws ContatoNaoEncontradoException, UsuarioNaoEncontradoException{
		this.repositorio.removerContato(login, contato);
		this.repositorio.removerContato(contato, login);
	}

	//loga o usuario
	private String[] logar(String login, String senha, InetAddress endereco, int porta) throws UsuarioNaoEncontradoException, SenhaInvalidaException{
		String [] retorno = new String[repositorio.getNumContatos(login)];
		String [] listaContatos = repositorio.getContatos(login, senha);
		for (int i = 0; i < retorno.length; i++) {
			retorno[i]=listaContatos[i];
		}
		setOnline(login, endereco, porta);
		return retorno;
	}

	//cadastra o usuario e devolte o resultado
	private String cadastro(String login, String senha){
		String cadastro = null;
		gui.adicionarEntrada(login, "-", "offline");
		try {
			cadastrarUsuario(login, senha);
			cadastro = "Pronto! Agora voc j pode utilizar o Sky-Pe a vontade.";
		} catch (JaCadastradoException e) {
			cadastro = "J existem alguem cadastrado com esse login, por favor tente outro ^^'";
		} catch (NaoPodeCadastrarMaisException e) {
			cadastro = "Sentimos muito em informar que chegamos ao numero maximo de usuarios ^^'";
		}
		return cadastro;
	}

	/*
	 * *************************
	 * METODOS DE ACESSO PUBLICO
	 * *************************
	 */

	//guarda o repositorio no arquivo dados.sky
	public void guardarDados(){
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(new File("dados.sky"));
			ObjectOutputStream dados = new ObjectOutputStream(fos);
			dados.writeObject(this.repositorio.getUsuarios());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} 

	}

	public DatagramPacket requisicao(DatagramPacket pedido){
		DatagramPacket resposta = null;

		byte[] requerimento = pedido.getData();
		byte operacao = requerimento[0];
		System.out.println(operacao);
		if(operacao == 0){//cadastro
			String login = new String(requerimento).substring(3, (3+requerimento[1]));
			System.out.println(login);
			String senha = new String(requerimento).substring((3+requerimento[1]), (3+requerimento[1]+requerimento[2]));
			System.out.println(senha);

			String mensagem = cadastro(login, senha);
			byte[] dados = mensagem.getBytes();
			resposta = new DatagramPacket(dados, dados.length, pedido.getAddress(), pedido.getPort());

		} else if (operacao == 1){//login
			
			byte erro = 0;				//byte que indica erro
			byte [] tamanho = null;		//tamanho do nome de cada contato
			String[] contatos;			//login dos contatos
			boolean[] onlines;			//valor que indica se o contato esta online ou nao
			int lista = 0;				//tamanho total o array de string
			byte[] online = null;		//valor que indica se o contato esta online ou nao em formato para ser enviado
			byte[]listaContatos = null;
			String login = new String(requerimento).substring(3, (3+requerimento[1]));
			System.out.println(login);
			String senha = new String(requerimento).substring((3+requerimento[1]), (3+requerimento[1]+requerimento[2]));
			System.out.println(senha);


			try {
				contatos = logar(login, senha, pedido.getAddress(), pedido.getPort());

				onlines = getOnline(contatos);
				tamanho = new byte[contatos.length];

				online = new byte[onlines.length];
				for (int i = 0; i < contatos.length; i++) {
					tamanho[i] = (byte) contatos[i].length();
					lista += contatos[i].length();
					if(onlines[i]){
						online[i] = 1;
					}else{
						online[i] = 0;
					}
				}
				listaContatos = new byte[lista];
				int k = 0;
				for (int i = 0; i < listaContatos.length; i++) {
					for (int j = 0; j < contatos[i].length(); j++) {
						listaContatos[k] = (byte) contatos[i].charAt(j);
						k++;
					}
				}
			} catch (UsuarioNaoEncontradoException e) {
				erro = 1;
			} catch (SenhaInvalidaException e) {
				System.out.println("senha errada maluco!");
				erro = 2;
			}
			byte aux = 0;
			if(tamanho != null){
				aux = (byte) tamanho.length;
			}
			byte[] dados = new byte[4+lista+aux+aux];
			dados[0] = erro;
			dados[1] = (byte) aux; //esse valor indicara o tamanho do primeiro e do ultimo array;
			//concatenar erro(1)+qtdTamanho(1)+tamanho(n)+listaContatos(lista)+online(n);
			//isso pode ser ate 2+99+12573+99=12773bytes, mas nao se preocupem, pois provavelmente ninguem 
			//tera 99 contatos e todos com 127 letras no login
			for (int i = 2; i < dados.length; i++) {
				if(i<2+aux){
					dados[i] = tamanho[i-2];
				}else if(i<2+aux+lista){
					dados[i] = listaContatos[i-2-aux];
				}else if(i<2+aux+lista+aux){
					dados[i] = online[i-2-aux-lista];
				}
			}
			resposta = new DatagramPacket(dados, dados.length, pedido.getAddress(), pedido.getPort());

		} else if (operacao == 2){//iniciar conexao
			try {
				logar(new String(requerimento).substring(4, 4+requerimento[1]), new String(requerimento).substring(4+requerimento[1], 4+requerimento[1]+requerimento[2]), pedido.getAddress(), pedido.getPort());
			} catch (UsuarioNaoEncontradoException e) {

			} catch (SenhaInvalidaException e) {

			}
			String contato = new String(requerimento).substring(4+requerimento[1]+requerimento[2]);
			int porta = getPorta(contato);
			InetAddress ip = getIP(contato);
			byte[] dados = new byte[9+contato.length()];
			for (int i = 0; i < 4; i++) {
				int offset = (dados.length - 1 - i) * 8;
				dados[i] = (byte) ((porta >>> offset) & 0xFF);
			}
			dados[4] = ip.getAddress()[0];
			dados[5] = ip.getAddress()[1];
			dados[6] = ip.getAddress()[2];
			dados[7] = ip.getAddress()[3];
			dados[8] = (byte) contato.length();

			for (int i = 9; i < dados.length; i++) {
				dados[i] = (byte) contato.charAt(i-9);
			}


			resposta = new DatagramPacket(dados, dados.length, pedido.getAddress(), pedido.getPort());

		} else if (operacao == 3){//adicionar contato
			byte[] erro = {0};
			String login = new String(requerimento).substring(4, 4+requerimento[1]);
			try {
				logar(login, new String(requerimento).substring(4+requerimento[1], 4+requerimento[1]+requerimento[2]), pedido.getAddress(), pedido.getPort());
				String contato = new String(requerimento).substring(4+requerimento[1]+requerimento[2]);
				adicionarContato(login, contato);
			} catch (UsuarioNaoEncontradoException e) {
				erro[0] = 1;
			} catch (SenhaInvalidaException e) {
				erro[0] = 2;
			} catch (JaEhContatoException e) {
				erro[0] = 3;
			} catch (NaoPodeAdicionarMaisException e) {
				erro[0] = 4;
			}

			resposta = new DatagramPacket(erro, 1, pedido.getAddress(), pedido.getPort());

		} else if (operacao == 4){//remover contatos
			byte[] erro = {0};
			String login = new String(requerimento).substring(4, 4+requerimento[1]);
			try {
				logar(login, new String(requerimento).substring(4+requerimento[1], 4+requerimento[1]+requerimento[2]), pedido.getAddress(), pedido.getPort());
				String contato = new String(requerimento).substring(4+requerimento[1]+requerimento[2]);
				removerContato(login, contato);
			} catch (UsuarioNaoEncontradoException e) {
				erro[0] = 1;
			} catch (SenhaInvalidaException e) {
				erro[0] = 2;
			} catch (ContatoNaoEncontradoException e) {
				erro[0]=5;
			}

			resposta = new DatagramPacket(erro, 1, pedido.getAddress(), pedido.getPort());

		} else if (operacao == 5){//desconectar usuario
			setOffline(new String(requerimento).substring(4, 4+requerimento[1]));
			byte[]erro= {0};
			resposta = new DatagramPacket(erro, 1, pedido.getAddress(), pedido.getPort());
		} 
		return resposta;
	}


	@Override
	public String[] usuariosCadastrados() {
		return this.usuarios;
	}

	@Override
	public boolean[] statusUsuario() {
		return this.usariosLogados;
	}

	@Override
	public InetAddress[] enderecoUsuario() {
		return this.endereco;
	}

	@Override
	public Integer[] portasUsuarios() {
		return this.portas;
	}

	/*
	 * ***************
	 * TESTE DE CLASSE
	 * ***************
	 */

	public static void main(String[] args) throws JaCadastradoException, NaoPodeCadastrarMaisException, NaoPodeAdicionarMaisException, UsuarioNaoEncontradoException, SenhaInvalidaException, ContatoNaoEncontradoException, JaEhContatoException {
		PistaPouso aeroporto = new PistaPouso();
		byte i = 0;
		try {
			final Servidor sky = new Servidor();
			//exibe a gui
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					ServidorGUI.runGUI(sky.gui);
				}
			});

			while(true){
				System.out.println(i+": ");
				DatagramPacket pedido = aeroporto.receber();
				byte op = pedido.getData()[0];
				pedido = sky.requisicao(pedido);
				aeroporto.enviar(pedido, op);
				i++;
			}


		}catch(Exception e) {
			e.printStackTrace();
		}
	}

}
/* O servidor podera receber varios tipos de pacotes, e dependendo desse tipo devera responder apropriadamente. 
 * Para identificar o tipo o pacote deve ter um numero identificador.
 * 
 * A porta do servidor : 20000.
 * 
 * A classe de recebimento dos pacotes fica responsavel por analisar e remover o checksum.
 * A classe de envio dos pacotes fica responsavel por adicionar 1byte de operacao e o checksum
 * 
 * O 1 byte de cada cabecario deve ter sempre o codigo da operacao
 **************************************************************************************************************************
 * 0: Cadastro: String cadastro(String login, String senha)
 * O sistema recebera o login e senha do usario e o adionara ao repositorio de usuarios, enviara depois uma mensagem de sucesso ou falha.
 * 
 * O pedido tem:
 * O segundo byte tem o tamanho do login
 * O terceiro byte tem o tamanho da senha
 * Login
 * Senha
 * 
 * A resposta tem:
 * 1 byte com o tamanho da string
 * Um string(Toda parte de dados sera um String) com a mensagem de sucesso ou erro.
 * 
 **************************************************************************************************************************
 * 1: Login/atualizar lista de contatos:
 * O sistema recebera o login, senha do usuario e o endereco dele. Respondera primeiramente com a lista de contatos
 *  e um array de booleanos que indica os contatos online.
 * Tambem pode servir com a utilidade de atualizar a lista de contatos e aqueles que estao online.
 * O cliente devera fazer isso ao menos a cada um minuto, mas o usuario podera atualizar a lista por ele mesmo.
 * 
 * O pedido tem:
 * O segundo byte tem o tamanho do login
 * O terceiro byte tem o tamanho da senha
 * Login
 * Senha
 * 
 * A resposta tem:
 * 1 byte representando se houve erro (0: tudo ok, 1: usuario nao cadastrado, 2: senha invalida)
 * 1 byte com o tamanho do array que diz o tamnho do login de cada contato
 * 1 array de byte com o tamanho do nome de cada contato
 * 1 array de String com os nomes do contato (pode ser nulo)
 * 1 array de booleanos mostrando os contatos online (pode ser nulo)
 **************************************************************************************************************************
 * 2: Usuario iniciando conexao com outro usuario
 * O sistema recebera  o login e senha do usuario e o login do contato. Se estiver disponivel devolvera o endereo de conexao do contato desejado.
 * 
 * O pedido tem:
 * O segundo byte tem o tamanho do login
 * O terceiro byte tem o tamanho da senha
 * O quarto byte tem o tamnho do login do contato
 * Login
 * Senha
 * Login do contato
 * 
 * A resposta tem:
 * 4 bytes que serao o numero da porta do contato
 * 4 bytes que serao o ip do contato
 * 1 byte com tamanho do login do contato
 * Login contato
 **************************************************************************************************************************
 * 3: Usuario adicionando contato
 * O sistema recebe o login, senha e login do contato e adiciona o contato a lista de contatos do usuario e vice versa.
 * 
 * O pedido tem:
 * O segundo byte tem o tamanho do login
 * O terceiro byte tem o tamanho da senha
 * O quarto byte tem o tamnho do login do contato
 * Login
 * Senha
 * Login do contato
 * 
 * A resposta tem:
 * 1 byte representando o erro.
 * 
 **************************************************************************************************************************
 * 4: Usuario removendo contato
 * similar a adicionar, mas remove.
 **************************************************************************************************************************
 * 5: Desconectar usuario
 * O sistema recebe a senha e login do usuario deixa-o em offline.
 * 
 * O pedido tem:
 * O segundo byte tem o tamanho do login
 * O terceiro byte tem o tamanho da senha
 * Login
 * Senha
 * 
 * A resposta tem: 
 * 1 byte representando o erro.
 */
