package servidor.gui;

import java.awt.Dimension;
import java.util.HashMap;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

public class ServidorGUI extends JPanel {
	private static final long serialVersionUID = -855685353922025310L;

	private JTable tabela;
	private ModeloTabelaServidor modeloTabela;
	private int indiceEntradas;
	private HashMap<String, Integer> mapaEntradas;

	public ServidorGUI() {
		indiceEntradas = 0;
		mapaEntradas = new HashMap<String, Integer>();
		modeloTabela = new ModeloTabelaServidor();
		tabela = new JTable(modeloTabela);
		tabela.setPreferredScrollableViewportSize(new Dimension(500, 160));
		tabela.setFillsViewportHeight(true);
		JScrollPane scrollPane = new JScrollPane(tabela);
		add(scrollPane);
	}

	public void adicionarEntrada(String usuario, String ip, String status) {
		if(!mapaEntradas.containsKey(usuario)) {
			//adiciona nova entrada se o usuario ainda nao estiver na lista
			modeloTabela.setValueAt(usuario, indiceEntradas, 0);
			modeloTabela.setValueAt(ip, indiceEntradas, 1);
			modeloTabela.setValueAt(status, indiceEntradas, 2);
			mapaEntradas.put(usuario, indiceEntradas);
			indiceEntradas++;
		}
		else {
			//se o usuario ja esta na tabela, apenas modifica os seus dados
			modificarEntrada(usuario, ip, status);
		}
	}

	public void modificarEntrada(String usuario, String novoIP, String novoStatus) {
		if(mapaEntradas.containsKey(usuario)) {
			int indice = mapaEntradas.get(usuario);
			modeloTabela.setValueAt(usuario, indice, 0);
			modeloTabela.setValueAt(novoIP, indice, 1);
			modeloTabela.setValueAt(novoStatus, indice, 2);
		}
		else {
			System.err.println("Usuario "+usuario+" nao consta na tabela!");
		}
	}

	public void removerEntrada(String usuario) {
		//substitui a entrada que se quer remover pela ultima e remove a ultima entrada
		Integer indice = mapaEntradas.get(usuario);
		if(indice != null) {
			//substitui os dados da entrada pelos dados da ultima linha da tabela
			modeloTabela.setValueAt(modeloTabela.getValueAt(indiceEntradas-1, 0), indice, 0);
			modeloTabela.setValueAt(modeloTabela.getValueAt(indiceEntradas-1, 1), indice, 1);
			modeloTabela.setValueAt(modeloTabela.getValueAt(indiceEntradas-1, 2), indice, 2);
			
			//apaga os dados na ultima linha da tabela
			modeloTabela.setValueAt("", indiceEntradas-1, 0);
			modeloTabela.setValueAt("", indiceEntradas-1, 1);
			modeloTabela.setValueAt("", indiceEntradas-1, 2);
			
			indiceEntradas--;
			mapaEntradas.remove(usuario);
		}
		else
			System.err.println("Impossivel remover: entrada nao existe!");
	}

	public static void runGUI(ServidorGUI gui) {
		JFrame frame = new JFrame("SkyPE");

		ServidorGUI contentPane = gui;
		contentPane.setOpaque(true);
		frame.setContentPane(contentPane);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setResizable(false);
		frame.pack();
		frame.setLocationRelativeTo(frame.getParent());
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				//runGUI();
			}
		});
	}

}

class ModeloTabelaServidor extends AbstractTableModel {
	private String[] colunas = {"Usuario", "Endereco IP", "Status"};
	private Object[][] dados;

	public ModeloTabelaServidor() {
		dados = new Object[10][3];
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = -1613375188464144267L;

	@Override
	public int getColumnCount() {
		return colunas.length;
	}

	@Override
	public int getRowCount() {
		return dados.length;
	}

	@Override
	public String getColumnName(int col) {
		return colunas[col];
	}

	@Override
	public Object getValueAt(int linha, int coluna) {
		return dados[linha][coluna];
	}

	public void setValueAt(Object value, int linha, int coluna) {
		dados[linha][coluna] = value;
		fireTableCellUpdated(linha, coluna);
	}
}
