package socketRTP;

/**
 * 
 * Essa classe implementa os servios de udio bsicos necessrios para a aplicao. O AudioCapture
 * faz basicamente a gravao e a execuo de udio. O udio a ser gravado ou executado deve estar
 * numa classe do tipo ByteArrayOutputStream.
 * 
 */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;

public class AudioService {
	protected boolean recording;
	protected boolean playing;
	Thread captureThread;
	Thread playThread;
	ByteArrayOutputStream audioOut;

	/**
	 * 
	 * Inicia a gravao de udio em um ByteArrayOutputStream
	 * 
	 */
	public void startRecord() {
		try {

			final AudioFormat format = getFormat();
			DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);
			final TargetDataLine line = (TargetDataLine) AudioSystem
					.getLine(info);
			line.open(format);
			line.start();

			Runnable runner = new Runnable() {
				int bufferSize = (int) format.getSampleRate()
						* format.getFrameSize();
				byte buffer[] = new byte[bufferSize];

				public void run() {
					audioOut = new ByteArrayOutputStream();
					recording = true;
					try {
						while (recording) {
							int count = line.read(buffer, 0, buffer.length);
							if (count > 0) {
								audioOut.write(buffer, 0, count);
							}
						}
						audioOut.close();
						line.close();
					} catch (IOException e) {
						System.err.println("I/O problems: " + e);
						System.exit(-1);
					}
				}
			};
			captureThread = new Thread(runner);
			captureThread.start();

		} catch (LineUnavailableException e) {
			System.err.println("Line unavailable: " + e);
			System.exit(-2);
		}
	}

	/**
	 * 
	 * Inicia a execuo de udio.
	 * 
	 * @param audioIn
	 *            O udio armazenado que deve ser executado.
	 * 
	 */
	public void startPlay(ByteArrayOutputStream audioIn) {
		try {
			byte audio[] = audioIn.toByteArray();
			InputStream input = new ByteArrayInputStream(audio);
			final AudioFormat format = getFormat();
			final AudioInputStream ais = new AudioInputStream(input, format,
					audio.length / format.getFrameSize());
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
			final SourceDataLine line = (SourceDataLine) AudioSystem
					.getLine(info);
			line.open(format);
			line.start();

			Runnable runner = new Runnable() {
				int bufferSize = (int) format.getSampleRate()
						* format.getFrameSize();
				byte buffer[] = new byte[bufferSize];

				public void run() {
					try {
						int count;
						playing = true;
						while ((count = ais.read(buffer, 0, buffer.length)) != -1
								&& playing) {
							if (count > 0) {
								line.write(buffer, 0, count);
							}
						}
						line.drain();
						line.close();
					} catch (IOException e) {
						System.err.println("I/O problems: " + e);
						System.exit(-3);
					}
				}
			};
			playThread = new Thread(runner);
			playThread.start();
		} catch (LineUnavailableException e) {
			System.err.println("Line unavailable: " + e);
			System.exit(-4);
		}
	}

	/**
	 * 
	 * Classe interna para obter as propriedades do udio.
	 * 
	 * @return O formato de udio.
	 * 
	 */
	private AudioFormat getFormat() {
		float sampleRate = 8000;
		int sampleSizeInBits = 8;
		int channels = 1;
		boolean signed = true;
		boolean bigEndian = true;
		return new AudioFormat(sampleRate, sampleSizeInBits, channels, signed,
				bigEndian);
	}

	/**
	 * 
	 * Para a gravao em andamento e retorna o udio gravado at o momento.
	 * 
	 * @return O udio que j foi gravado.
	 * 
	 */
	public ByteArrayOutputStream stopRecord() {
		try {
			recording = false;
			captureThread.join();
		} catch (InterruptedException e) {
			System.exit(1);
		}
		return audioOut;
	}

	/**
	 * 
	 * Termina a execuo de udio em andamento.
	 * 
	 */
	public void stopPlay() {
		try {
			playing = false;
			playThread.join();
		} catch (InterruptedException e) {
			System.exit(1);
		}
	}

}
